import os
import sys

SCRIPT_DIR = os.path.dirname(os.path.abspath(__file__))
sys.path.append(os.path.dirname(SCRIPT_DIR))

from common import gitlab_helpers
from common import git_helpers
from common import version_helpers
from common import sentry_helpers

if __name__ == "__main__":
    print(sys.argv)

    for key, value in os.environ.items():
        print("'{}': '{}'".format(key, value))

    sentry_repository_path = os.environ['SENTRY_REPOSITORY_PATH']
    project_name = os.environ['CI_PROJECT_TITLE']
    sentry_token = os.environ['SENTRY_TOKEN']
    api_token = os.environ['API_ACCESS_TOKEN']
    project_id = os.environ['CI_PROJECT_ID']
    branch_name = os.environ['CI_COMMIT_BRANCH']

    try:
        release = os.environ['RELEASE']
    except:
        release = ""

    # Получение последней версии
    try:
        last_version = os.environ['LAST_RELEASE_VERSION']
    except:
        last_version = "0.1.0"

    # получение последнего коммита на момент сохранения последней версии
    try:
        last_hexsha = os.environ[gitlab_helpers.BRANCH_LAST_RELEASE_HASH]
    except:
        try:
            last_hexsha = os.environ[gitlab_helpers.LAST_RELEASE_HASH]
        except:
            last_hexsha = ""

    # определения типа релиза
    if release == "main" or branch_name == "main":
        release_type = "main"
    elif release == "dev" or branch_name == "dev":
        release_type = "dev"
    else:
        release_type = ""

    # если релиз main значит это выпуск в продакшн уже существующей версии
    if release_type == "main":
        print("Расчет продакшн версии.")

        # в этом случае нужно:
        #   1. взять уже существующую версию
        version = last_version
        #   2. посчитать новые коммиты 
        commits = git_helpers.get_new_commits(
            sentry_repository_path,
            project_name,
            last_hexsha)
        #   4. задать информацию о релизе (версия, последний коммит)
        gitlab_helpers.set_release_values(
            api_token,
            project_id,
            version,
            git_helpers.get_new_hash())
        #   5. выпустить тег в гитлабе
        gitlab_helpers.create_tag(
            api_token,
            project_id,
            branch_name,
            version,
            commits)
        #   6. выпустить релиз в гитлабе
        gitlab_helpers.create_release(
            api_token,
            project_id,
            version)
        #   7. выпустить релиз в сентри
        sentry_helpers.create_release(
            sentry_token,
            project_name,
            version,
            commits)

    # если релиз main значит это выпуск в продакшн уже существующей версии
    elif release_type == "dev":
        print("Расчет версии для разработчиков.")

        # в этом случае нужно:
        #   1. расчитать новую версию
        version = version_helpers.get_new_version(
            last_version,
            last_hexsha)
        #   2. посчитать новые коммиты 
        commits = git_helpers.get_new_commits(
            sentry_repository_path,
            project_name,
            last_hexsha)
        #   3. задать информацию о релизе (версия, последний коммит)
        gitlab_helpers.set_release_values(
            api_token,
            project_id,
            version,
            git_helpers.get_new_hash())

    else:
        print("Без расчета новой версии.")
        version = last_version
    

    print("next-version: {0}".format(version))

    with open('.next-version', 'w') as version_file:
        version_file.write(version)
